# 04 Intra- and Inter-software test
You can investigate the structure equivalency generated by a software pakcage using 600 CIF files taken from the Crystallography Open Database (COD) database [http://www.crystallography.net/cod/].

:warning: To reproduce the results in the paper, please skip the autosub.py part because the script uses random numbers.

## Generating CIFs with partial occupations
autosub.py is a tool to generate CIFs with partial occupations. You can specify the maxmum substitution numbers in autosub.py

```pyhthon:bench.py
CONFIG_IRREDUCIBLE_MAX = 3e3
CONFIG_IRREDUCIBLE_MIN = 1e1
TOTAL_SUBSTITUTION_MIN = 1e1
TOTAL_SUBSTITUTION_MAX = 1e4
```

Before the benchmark test, you should establish your python enviroment in which all the depenedent python packages are installed.

```bash:
conda activate shry
```

run autosub.py (sequentially)

```bash:
sg=XX
python ./autosub.py SG${sg}> out_autosub_SG${sg}
```

You can also submit the jobs simultaneously see. ``submit_autosub.sh``.

Rename the xls file for the proceeding calculations

```bash:
cp autosub_SG${sg}.xls full_consistency_check.xls  # Rename the xls file for the proceeding calculations
```

You can combine xls files if you run several SGs.

```bash:
python combine_xls.py  # combine all autosub_SG${sg} file
cp autosub_SG_all.xls full_consistency_check.xls  # Rename the xls file for the proceeding calculations
```

Finally, you should fix HM symbols of the generated CIFs with partial occupations using ``HM_fix.sh``.

```bash:
./HM_fix.sh
```

## Benchmark test

You can check if the numbers of symmetry-equivalent structures are consistent with expected values.

```pyhthon:bench.py
BABEL=False # True for supercell ver.1.2, False for that >= ver.2.0
SUPERCELL_PATH="your path to supercell BINARY"
SHRY_PATH = "your path to shry DIRECTORY"

SUPERCELL_FLAG=True # if false, supercell jobs are skipped.
SHRY_FLAG=True # if false, shry jobs are skipped.
```

```bash:
python bench.py benchmark_SG_all.xls SG_all > out_bench_SG_all # run test
```

The output of bench.py is written in ``out_bench_SG_all``. The outputs of shry and supercell are stored in ``stdout`` and ``stderr`` directories.

## Generating symmetry-inequivalent structures

You can generate symmetry-iniquivalent structures using ``prep_structure.py``.

```pyhthon:prep_structure.py
BABEL=False # True for supercell ver.1.2, False for that >= ver.2.0
SUPERCELL_PATH="your path to supercell BINARY"
SHRY_PATH="your path to shry DIRECTORY"
```

The script generates CIF files of symmetry inequivalent structures and zip them.

```bash:
xls_file="full_consistency_check.xls"
python prep_structure.py $xls_file
```

symmetry inequivalent structures are generated by supercell and/or shry, and then zipped as ``supercell-SGXXX_YYY.zip`` and ``shry-SGXXX_YYY.zip``.

## Investigation of full consistency check

You can do intra-software and inter-software tests using ``check_full_serial.py`` or ``check_full_mpi.py``.

```pyhthon:check_full_XXX.py
check_internal_redundancy=False  # intra-software check
check_consistency=False  # inter-software check
```

You can run consistency check.

```bash:
xls_file="full_consistency_check.xls"
python ./check_full_serial.py $xls_file # serial version
mpirun -np 1536 python ./check_full_mpi.py $xls_file # MPI version
```

The found pairs of CIF files are saved as csv files, ``SGXXX_YYY.cif_pairs.csv``.
